@@ -212,40 +212,42 @@ struct CUSTOM_API GDLInterpreterTokenTypes {
 		CONSTANT_BIN_ULONG64 = 201,
 		CONSTANT_BIN_UI = 202,
 		CONSTANT_BIN_UINT = 203,
-		ASTERIX = 204,
-		DOT = 205,
-		STRING_LITERAL = 206,
-		POW = 207,
-		MATRIX_OP1 = 208,
-		MATRIX_OP2 = 209,
-		PLUS = 210,
-		MINUS = 211,
-		LTMARK = 212,
-		GTMARK = 213,
-		LOG_NEG = 214,
-		LOG_AND = 215,
-		LOG_OR = 216,
-		QUESTION = 217,
-		STRING = 218,
-		INCLUDE = 219,
-		EOL = 220,
-		W = 221,
-		D = 222,
-		L = 223,
-		H = 224,
-		O = 225,
-		B = 226,
-		EXP = 227,
-		DBL_E = 228,
-		DBL = 229,
-		CONSTANT_OR_STRING_LITERAL = 230,
-		COMMENT = 231,
-		END_MARKER = 232,
-		WHITESPACE = 233,
-		SKIP_LINES = 234,
-		CONT_STATEMENT = 235,
-		END_OF_LINE = 236,
-		MAX_TOKEN_NUMBER = 237,
+		CONSTANT_CMPLX_I = 204,
+		CONSTANT_CMPLXDBL_I = 205,
+		ASTERIX = 206,
+		DOT = 207,
+		STRING_LITERAL = 208,
+		POW = 209,
+		MATRIX_OP1 = 210,
+		MATRIX_OP2 = 211,
+		PLUS = 212,
+		MINUS = 213,
+		LTMARK = 214,
+		GTMARK = 215,
+		LOG_NEG = 216,
+		LOG_AND = 217,
+		LOG_OR = 218,
+		QUESTION = 219,
+		STRING = 220,
+		INCLUDE = 221,
+		EOL = 222,
+		W = 223,
+		D = 224,
+		L = 225,
+		H = 226,
+		O = 227,
+		B = 228,
+		EXP = 229,
+		DBL_E = 230,
+		DBL = 231,
+		CONSTANT_OR_STRING_LITERAL = 232,
+		COMMENT = 233,
+		END_MARKER = 234,
+		WHITESPACE = 235,
+		SKIP_LINES = 236,
+		CONT_STATEMENT = 237,
+		END_OF_LINE = 238,
+		MAX_TOKEN_NUMBER = 239,
 		NULL_TREE_LOOKAHEAD = 3
 	};
 #ifdef __cplusplus
