@@ -21,12 +21,10 @@
 #include <string>
 #include <fstream>
 #include <limits>
-
+#include "gdleventhandler.hpp" //for gdlwait_responsive
 #include "dinterpreter.hpp"
 #include "basic_pro_jmg.hpp"
-
-//#define GDL_DEBUG
-//#undef GDL_DEBUG
+#include "gdlhelp.hpp" //for dlmInfo
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #   include "gtdhelper.hpp" // just a workaround, using QueryPerformanceCounter is better
@@ -35,63 +33,265 @@ typedef HMODULE handle_t;
 typedef void* handle_t;
 #endif
 
+#include "export.hpp"
+
+static SizeT increment=0;
+
 namespace lib {
+    using namespace std;
 
-  using namespace std;
+    //search in AllDLMSymbols all functions/procs referring to this dlm name, and clears them to indicate that the "Loaded DLM" message has already been activated.
+    //mimics IDL's behaviour although we do not use the same logic, the dlls are loaded when the dlm is read, not when an entry is called.
+    void ClearDlmStringReference(std::string &s){
+      std::string local_s=s;
+      for (auto i=0; i< AllDLMSymbols.size() ;++i) if (AllDLMSymbols[i].second.size() && AllDLMSymbols[i].second == local_s) AllDLMSymbols[i].second.clear();
+    }
+    std::string MyFunName(size_t t){
+        for (std::map<const char*, void*>::iterator it = SysFunDefinitions.begin(); it != SysFunDefinitions.end(); ++it) if ((size_t)(it->second)==t) return it->first;
+        return "Unknown";
+    }
+    std::string MyProName(size_t t){
+        for (std::map<const char*, void*>::iterator it = SysProDefinitions.begin(); it != SysProDefinitions.end(); ++it) if ((size_t)(it->second)==t) return it->first;
+        return "Unknown";
+    }
+  
+  BaseGDL* CallDllFunc(EnvT* e) {TRACE_ROUTINE(__FUNCTION__, __FILE__, __LINE__)
+  SizeT pos=(SizeT)(static_cast<DLibFun*> (e->GetPro())->GetFunAddress());
+  EXPORT_SYSRTN_FUN calldllfunc= (EXPORT_SYSRTN_FUN) AllDLMSymbols[pos].first;
+    if ((AllDLMSymbols[pos].second).size()) {
+      std::cout<<"Loaded DLM: "<<AllDLMSymbols[pos].second<<std::endl;
+      ClearDlmStringReference(AllDLMSymbols[pos].second);
+    }
+    int argc = e->NParam();
+    EXPORT_VPTR argv[argc];
+    bool tempo[argc];
+#ifdef GDL_DEBUG
+    std::cerr<<"FUNCTION "<<MyFunName((size_t)calldllfunc)<<std::endl;
+#endif
+    for (auto i = 0; i < argc; ++i) {
+      tempo[i] = (e->GetString(i).find('>') != std::string::npos);      // tells if input parameter is temporary (expression)
+      argv[i] = GDL_ToVPTR(e->GetPar(i), tempo[i]);
+      if (!tempo[i]) PassedVariables[argv[i]]=e->GetString(i) ; //add to list of passed NAMED variables
+#ifdef GDL_DEBUG
+      std::cerr << e->GetString(i) << "="; if (e->GetPar(i)) e->GetPar(i)->ToStream(std::cerr); else std::cerr<<"<Undefined>"; std::cerr<<std::endl;
+#endif
+    }
+    char *argk = NULL;
+    // keywords are passed as _REF_EXTRA struct, we just populate argk with our GDL_KEYWORDS_LIST struct
+    SizeT nkw = 0;
+    GDL_PASS_KEYWORDS_LIST passed;
+    GDL_KEYWORDS_LIST* kws = NULL;
+    DStringGDL* refextra;
+    if (e->GetPro()->NKey() > 0) {
+      //pass in "argk" as a_REF_EXTRA 
+      //we cannot directly use the _EXTRA mechanism as some passed values should be writeable or are not defined at the time of calling this function
+      if (e->KeywordPresentAndDefined(0)) {
+        refextra = e->GetKWAs<DStringGDL>(0);
+        nkw = refextra->N_Elements();
+        kws = (GDL_KEYWORDS_LIST*) calloc(nkw,sizeof (GDL_KEYWORDS_LIST));
+        for (auto i = 0; i < nkw; ++i) {
+          kws[i].name = (*refextra)[i].c_str();
+          BaseGDL** gvarp = e->GetRefExtraListPtr((*refextra)[i]); //check as Ptr as the variable may not exist
+          if (gvarp == NULL) {
+            kws[i].readonly = 1;
+            //pass the variable anyway using GetRefExtraList
+            BaseGDL* gvar = e->GetRefExtraList((*refextra)[i]);
+            // Add var name to PassedVariables
+            kws[i].varname = e->Caller()->GetString(gvar);
+            kws[i].varptr = gvar;
+            if (gvar == NULL) kws[i].type = GDL_TYP_UNDEF;
+          } else {
+            kws[i].readonly = 0;
+            kws[i].varptr = *gvarp;
+            if (*gvarp == NULL) kws[i].type = GDL_TYP_UNDEF;
+            kws[i].varname = e->Caller()->GetString(*gvarp);
+          }
+#ifdef GDL_DEBUG
+          std::cerr<<kws[i].name<<": "<<kws[i].varname<<"="; (kws[i].varptr)->ToStream(std::cerr);
+#endif
+        }
+      }
+      passed.npassed = nkw;
+      passed.passed = kws;
+      argk = (char*) (&passed);
+    }
+    EXPORT_VPTR ret;
+    try{
+      ret = calldllfunc(argc, argv, argk);
+    } catch (GDLException& e) { throw e;} 
+      catch (...) {e->Throw("error in DLM code / unsupported procedure, returning.");
+    }
+    if (ret->type == GDL_TYP_UNDEF) e->Throw("Variable is undefined: <UNDEFINED>.");
+    BaseGDL* back = VPTR_ToGDL(ret, true); //protect data
 
-  void CleanupProc( DLibPro* proc ) {
-    auto it = libProList.begin();
-    auto itE = libProList.end();
-    for( ; it != itE; it++ ) {
-      if( *it == proc ) break;
+    for (auto i = 0; i < argc; ++i) {
+      if (!tempo[i]) e->SetPar(i, VPTR_ToGDL(argv[i]));
+      if (!tempo[i]) PassedVariables.erase(argv[i]); //remove map entry
     }
-    if( it < itE ) {
-      delete *it;
-      libProList.erase( it );
+    //check if some argk keywords have been returned too. A real variable must be associated to be replaced in return
+    for (auto i = 0; i < nkw; ++i) {
+        if (kws[i].out != NULL) {
+        PassedVariables.erase(kws[i].out); //remove map entry
+        BaseGDL** gvarp = e->GetRefExtraListPtr((*refextra)[i]); //Ptr as the variable may not exist
+        if (gvarp) { //replace parameter's value
+          GDLDelete(*gvarp);
+          *gvarp = (BaseGDL*) VPTR_ToGDL((EXPORT_VPTR) (kws[i].out));
+        } else {
+          e->Throw("Unexpected error, variable not existing. Please report.");
+        }
+      }
     }
+    GDL_FreeResources() ;
+    return back;
   }
 
+  void CallDllPro(EnvT* e) {TRACE_ROUTINE(__FUNCTION__, __FILE__, __LINE__)
+  SizeT pos=(SizeT)(static_cast<DLibPro*> (e->GetPro())->GetProAddress());
+    EXPORT_SYSRTN_PRO calldllpro = (EXPORT_SYSRTN_PRO) AllDLMSymbols[pos].first;
+    if ((AllDLMSymbols[pos].second).size()) {
+      std::cout<<"Loaded DLM: "<<AllDLMSymbols[pos].second<<std::endl;
+      ClearDlmStringReference(AllDLMSymbols[pos].second);
+    }
+    int argc = e->NParam();
+    EXPORT_VPTR argv[argc];
+    bool tempo[argc];
+#ifdef GDL_DEBUG
+    std::cerr<<"PROCEDURE "<<MyProName((size_t)calldllpro)<<std::endl;
+#endif
+    for (auto i = 0; i < argc; ++i) {
+      tempo[i] = (e->GetString(i).find('>') != std::string::npos);      // tells if input parameter is temporary (expression)
+#ifdef GDL_DEBUG
+      std::cerr << e->GetString(i) << "=";
+      if (e->GetPar(i)) e->GetPar(i)->ToStream(std::cerr);
+      else std::cerr << "<Undefined>";
+      std::cerr << std::endl;
+#endif
+      argv[i] = GDL_ToVPTR(e->GetPar(i), tempo[i]);
+      if (!tempo[i]) PassedVariables[argv[i]]=e->GetString(i) ; //add to list of passed NAMED variables
+    }
+    char *argk = NULL;
+    // keywords are passed as _REF_EXTRA struct, we just populate argk with our GDL_KEYWORDS_LIST struct
+    SizeT nkw = 0;
+    GDL_PASS_KEYWORDS_LIST passed;
+    GDL_KEYWORDS_LIST* kws = NULL;
+    DStringGDL* refextra;
+    if (e->GetPro()->NKey() > 0) {
+      //pass in "argk" as a_REF_EXTRA 
+      //we cannot directly use the _EXTRA mechanism as some passed values should be writeable or are not defined at the time of calling this function
+      if (e->KeywordPresentAndDefined(0)) {
+        refextra = e->GetKWAs<DStringGDL>(0);
+        nkw = refextra->N_Elements();
+        kws = (GDL_KEYWORDS_LIST*) calloc(nkw,sizeof (GDL_KEYWORDS_LIST));
+        for (auto i = 0; i < nkw; ++i) {
+          kws[i].name = (*refextra)[i].c_str();
+          BaseGDL** gvarp = e->GetRefExtraListPtr((*refextra)[i]); //check as Ptr as the variable may not exist
+          if (gvarp == NULL) {
+            kws[i].readonly = 1;
+            //pass the variable anyway using GetRefExtraList
+            BaseGDL* gvar = e->GetRefExtraList((*refextra)[i]);
+            // Add var name to PassedVariables
+            kws[i].varname=e->Caller()->GetString(gvar);
+            kws[i].varptr = gvar;
+            if (gvar == NULL) kws[i].type = GDL_TYP_UNDEF;
+          } else {
+            kws[i].readonly = 0;
+            kws[i].varptr = *gvarp;
+            if (*gvarp == NULL) kws[i].type = GDL_TYP_UNDEF;
+            kws[i].varname=e->Caller()->GetString(*gvarp);
+          }
+#ifdef GDL_DEBUG
+          std::cerr<<kws[i].name<<": "<<kws[i].varname<<"="; (kws[i].varptr)->ToStream(std::cerr);
+#endif
+        }
+      }
+      passed.npassed = nkw;
+      passed.passed = kws;
+      argk = (char*) (&passed);
+    }
+    try{
+      calldllpro(argc, argv, argk);
+    } catch (GDLException& e) { throw e;} 
+      catch (...) {e->Throw("error in DLM code / unsupported procedure, returning.");}
+    for (auto i = 0; i < argc; ++i) {
+      if (!tempo[i]) e->SetPar(i, VPTR_ToGDL(argv[i]));
+      if (!tempo[i]) PassedVariables.erase(argv[i]); //remove map entry
+    }
+    //check if some argk keywords have been returned too. A real variable must be associated to be replaced in return
+    for (auto i = 0; i < nkw; ++i) {
+      if (kws[i].out != NULL) {
+        BaseGDL** gvarp = e->GetRefExtraListPtr((*refextra)[i]); //Ptr as the variable may not exist
+        if (gvarp) {
+          GDLDelete(*gvarp);
+          *gvarp = (BaseGDL*) VPTR_ToGDL((EXPORT_VPTR) (kws[i].out));
+        } else {
+          e->Throw("Unexpected error, variable not existing. Please report.");
+        }
+      }
+    }
+    GDL_FreeResources() ;
+  }
+
+void CleanupProc( DLibPro* proc ) {
+// this does not work (double deletion at exit due to ~DllContainer() { unload( true ); } that already uses unlinksymbol.
+//    auto it = libProList.begin();
+//    auto itE = libProList.end();
+//    for( ; it != itE; it++ ) {
+//      if( *it == proc ) break;
+//    }
+//    if( it < itE ) {
+//        delete *it;
+//      libProList.erase( it );
+//      }
+    }
+
   void CleanupFunc( DLibFun* func ) {
-    auto it = libFunList.begin();
-    auto itE = libFunList.end();
-    for( ; it != itE; it++ ) {
-      if( *it == func ) break;
-    }
-    if( it < itE ) {
-      delete *it;
-      libFunList.erase( it );
-    }
+// this does not work (double deletion at exit due to ~DllContainer() { unload( true ); } that already uses unlinksymbol.
+//    auto it = libFunList.begin();
+//    auto itE = libFunList.end();
+//    for( ; it != itE; it++ ) {
+//      if( *it == func ) break;
+//    }
+//    if( it < itE ) {
+//      delete *it;
+//      libFunList.erase( it );
+//    }
   }
 
   struct DllContainer {
+    handle_t handle; // Handle to the linked DLL
+
+    set<string> my_procs; // list of procedures linked within this DLL
+    set<string> my_funcs; // list of functions linked within this DLL
+    set<handle_t> my_handles; // memorize handles to avoid calling IDL_Load() many times
+
+    static map<string, shared_ptr<DLibPro>> all_procs; // list of ALL procedures linked (using linkimage)
+    static map<string, shared_ptr<DLibFun>> all_funcs; // list of ALL functions linked (using linkimage)
+    static map<string, DllContainer> libs; // list of ALL DLLs linked (using linkimage)
+
+    bool isok=false;
+    std::string DllLoadErrorMessage;
+    bool addedToHelpDll=false;
+    bool IsOK(){return isok;}
+    bool IsHelpAdded(){return addedToHelpDll;}
+    void HelpAdded(bool b){addedToHelpDll=b;}
     DllContainer( const DllContainer& ) = delete;
     DllContainer( DllContainer&& rhs ) : handle(nullptr) { std::swap( handle, rhs.handle ); };
-    DllContainer( const string& fn ) : handle(nullptr) { load(fn); }
-    ~DllContainer() { unload( true ); }
+    DllContainer( const string& fn ) : handle(nullptr) { };//load(fn); }/// do not load at DllContainer definition! will be done better after.
+    ~DllContainer() { unload( true ); isok=false;}
+    
     void load( const string& fn ) {
       if( handle ) return;     // already loaded.
-      string msg;
 #if defined(_WIN32) && !defined(__CYGWIN__)
       WCHAR u_shrdimgName[255];
       MultiByteToWideChar(CP_ACP, 0, fn.c_str(), fn.length(), u_shrdimgName, 255);
       handle = LoadLibraryW(u_shrdimgName);
-      if( !handle ) {
-	msg = "Couldn't open " + fn;
-      }
+      if( !handle ) DllLoadErrorMessage = fn; else isok=true;
 #else
-      handle = dlopen(fn.c_str(), RTLD_LAZY);
-      if( !handle ) {
-	msg = "Couldn't open " + fn;
-	char* error = dlerror();
-	if( error ) {
-	  msg += string(": ") + error;
-	}
-      }
+      handle = dlopen(fn.c_str(), RTLD_LAZY|RTLD_GLOBAL);
+      if( !handle ) DllLoadErrorMessage = string(dlerror()); else isok=true;
 #endif
-      if( !handle ) {
-	throw runtime_error( msg );
-      }
     }
+    
     void unload( bool force=false ) {
       if( !force && !(my_procs.empty() && my_funcs.empty()) ) {
 	return;
@@ -110,13 +310,14 @@ namespace lib {
       if( handle ) dlclose(handle);
 #endif
       handle = nullptr;
+      isok=false;
     }
     static DllContainer& get( const string& fn ) {
       auto res = libs.emplace( std::pair<string,DllContainer>(fn, DllContainer(fn)) );
       DllContainer& l = res.first->second;
       if( ! l.isLoaded() ) {
-	l.load(fn);
-      }
+        l.load(fn);
+     }
       return l;
     }
     static void unload( const string& fn, bool force=false ) {
@@ -130,19 +331,65 @@ namespace lib {
       all_procs.clear();
       all_funcs.clear();
     }
-    void RegsisterSymbol( const string& lib_symbol, const string& proc_name, DLong funcType, DLong max_args=16, DLong min_args=0, const string keyNames[]=NULL ) {
+
+    void RegisterSymbolDefinedByIDL_Load(const string& dlm_name, const string& proc_name, DLong funcType, DLong max_args = 16, DLong min_args = 0, bool has_keys = false) {
+      if (!handle) {
+        throw runtime_error("Library not loaded!");
+      } else if (funcType < 0 || funcType > 1) {
+        throw runtime_error("Improper function type: " + to_string(funcType));
+      }
+      const char * name = proc_name.c_str();
+      if (funcType == 0) {
+        bool found = false;
+        for (std::map<const char*, void*>::iterator it = SysProDefinitions.begin(); it != SysProDefinitions.end(); ++it) {
+          if (strcmp(it->first, name) == 0) {
+            //find lib_symbol in SysPropDefinitions
+            RegisterMediatizedProc(dlm_name, it->second, proc_name, max_args, min_args, has_keys);
+            found = true;
+            break;
+          }
+        }
+        if (!found) throw runtime_error("DLM's IDL_Load() does not define procedure " + proc_name);
+        } else {
+        bool found = false;
+        for (std::map<const char*, void*>::iterator it = SysFunDefinitions.begin(); it != SysFunDefinitions.end(); ++it) {
+          if (strcmp(it->first, name) == 0) {
+            //find lib_symbol in SysFunDefinitions
+            RegisterMediatizedFunc(dlm_name, it->second, proc_name, max_args, min_args, has_keys);
+            found = true;
+            break;
+          }
+        }
+        if (!found) throw runtime_error("DLM's IDL_Load()  does not define function " + proc_name);
+        }
+      }
+    
+    void RegisterSymbol(const string & dlm_name, const string& entry_name, const string& proc_name, DLong funcType, DLong max_args=16, DLong min_args=0, bool has_keys=false) {
       if( !handle ) {
 	throw runtime_error( "Library not loaded!" );
       } else if( funcType < 0 || funcType>1 ) {
 	throw runtime_error( "Improper function type: "+to_string(funcType) );
       }
       if( funcType == 0 ) {
-	RegsisterProc( lib_symbol, proc_name, max_args, min_args, keyNames );
+	RegisterMediatizedProc( dlm_name, MakeTarget(entry_name,proc_name), proc_name, max_args, min_args, has_keys);
       } else {
-	RegsisterFunc( lib_symbol, proc_name, max_args, min_args, keyNames );
+	RegisterMediatizedFunc( dlm_name, MakeTarget(entry_name,proc_name), proc_name, max_args, min_args, has_keys);
       }
     }
-    void UnregsisterSymbol( const string& proc_name, DLong funcType ) {
+ 
+    void RegisterNativeSymbol( const string& entry_name, const string& proc_name, DLong funcType, DLong max_args=16, DLong min_args=0, string keyNames[]=NULL) {
+      if( !handle ) {
+	throw runtime_error( "Library not loaded!" );
+      } else if( funcType < 0 || funcType>1 ) {
+	throw runtime_error( "Improper function type: "+to_string(funcType) );
+      }
+      if( funcType == 0 ) {
+	RegisterNativeProc( entry_name, proc_name, max_args, min_args, keyNames);
+      } else {
+	RegisterNativeFunc( entry_name, proc_name, max_args, min_args, keyNames);
+      }
+    }
+    void UnregisterSymbol( const string& proc_name, DLong funcType ) {
       if( !handle ) {
 	throw runtime_error( "Library not loaded!" );
       } else if( funcType < 0 || funcType>1 ) {
@@ -156,23 +403,123 @@ namespace lib {
 	my_funcs.erase( proc_name );
       }
     }
-    template <typename T>
-    T LinkAs( const string& lib_symbol, const string& proc_name ) { 
+    //for IDL-compatible library, try IDL_Load, that may define useful variables or structures before the use of the othe functions/procedures inside.
+    int CallLoadToDefineEntryLocations(){
+      if (my_handles.count(handle) > 0) return 1; //already known
+      void* fPtr=NULL;
+      char* error = nullptr;
+#if defined(_WIN32) && !defined(__CYGWIN__)
+      fPtr = (void*) GetProcAddress( handle, "IDL_Load" );
+#else
+      error = dlerror();  // clear error
+      fPtr = dlsym( handle, "IDL_Load" );
+      error = dlerror();
+#endif
+      if(fPtr) { //IDL_Load() may be absent, no probs for us.
+        // if it is present, we call it. If it contains calls to IDL_SysRtnAdd(), then IDL_SysRtnAdd() will to load all the definitions, they will not be defined again.
+        int (*Call_IDL_Load)(void) = ( int (*)(void)) fPtr;
+        int ret ;
+        try{
+          ret = Call_IDL_Load();
+        } catch (GDLException ex) { throw GDLException("Error while calling IDL_Load(): "+ex.toString());}
+          catch (...) { throw GDLException("Error while calling IDL_Load()");}
+        if (ret) my_handles.insert(handle); // IDL_Load has been called
+        return ret;
+      }
+      return 0;
+    }
+   template <typename T>
+    T LinkAs(const string& lib_symbol, const string& proc_name) {
       T fPtr = nullptr;
       char* error = nullptr;
 #if defined(_WIN32) && !defined(__CYGWIN__)
-      fPtr = (T) GetProcAddress( handle, lib_symbol.c_str() );
+      fPtr = (T) GetProcAddress(handle, lib_symbol.c_str());
 #else
-      error = dlerror();  // clear error
-      fPtr = (T) dlsym( handle, lib_symbol.c_str() );
+      error = dlerror(); // clear error
+      fPtr = (T) dlsym(handle, lib_symbol.c_str());
       error = dlerror();
 #endif
+      if (error) { //try lowercase (DLMs are written in UPPERCASE)
+        std::string s=StrLowCase(lib_symbol);
+#if defined(_WIN32) && !defined(__CYGWIN__)
+      fPtr = (T) GetProcAddress( handle, s.c_str() );
+#else
+      error = dlerror();  // clear error
+      fPtr = (T) dlsym( handle, s.c_str() );
+      error = dlerror();
+#endif
+      }
+      if (error) { //try uppercase?
+        std::string s=StrUpCase(lib_symbol);
+#if defined(_WIN32) && !defined(__CYGWIN__)
+      fPtr = (T) GetProcAddress( handle, s.c_str() );
+#else
+      error = dlerror();  // clear error
+      fPtr = (T)  dlsym( handle, s.c_str() );
+      error = dlerror();
+#endif
+      }
+      if (error) {
+        throw runtime_error("Failed to register DLL-routine: " + proc_name + string(" -> ") + lib_symbol + string(" : ") + error);
+      }
+      return fPtr;
+    }
+    
+    void* MakeTarget( const string& lib_symbol, const string& proc_name ) { 
+      void* fPtr = nullptr;
+      char* error = nullptr;
+      // if 'lib_symbol' does not work, try both upcase and lowcase versions
+#if defined(_WIN32) && !defined(__CYGWIN__)
+      fPtr = (void*) GetProcAddress( handle, lib_symbol.c_str() );
+#else
+      error = dlerror();  // clear error
+      fPtr = dlsym( handle, lib_symbol.c_str() );
+      error = dlerror();
+#endif
+      // if 'lib_symbol' does not work, try both upcase and lowcase versions
+      if (error) { //try lowercase (DLMs are written in UPPERCASE)
+        std::string s=StrLowCase(lib_symbol);
+#if defined(_WIN32) && !defined(__CYGWIN__)
+      fPtr = (void*) GetProcAddress( handle, s.c_str() );
+#else
+      error = dlerror();  // clear error
+      fPtr = dlsym( handle, s.c_str() );
+      error = dlerror();
+#endif
+      }
+      if (error) { //try uppercase?
+        std::string s=StrUpCase(lib_symbol);
+#if defined(_WIN32) && !defined(__CYGWIN__)
+      fPtr = (void*) GetProcAddress( handle, s.c_str() );
+#else
+      error = dlerror();  // clear error
+      fPtr = dlsym( handle, s.c_str() );
+      error = dlerror();
+#endif
+      }
       if( error ) {
 	throw runtime_error( "Failed to register DLL-routine: " + proc_name + string(" -> ") + lib_symbol + string(" : ") + error );
       }
       return fPtr;
     }
-    void RegsisterProc( const string& lib_symbol, const string& proc_name, DLong max_args, DLong min_args, const string keyNames[] ) { 
+#define ADD_PROC_TO_INFOLIST_AND_INCREMENT_COUNTER std::pair <void*, std::string> newpair;\
+      newpair.first=target;\
+      newpair.second=dlm_name;\
+      AllDLMSymbols.push_back(newpair);
+
+    void RegisterMediatizedProc(const string& dlm_name, void* target, const string& proc_name, DLong max_args, DLong min_args, bool has_keys) {
+      if (all_procs.count(proc_name)) return;
+      // this method of 'cleaning' is excellent but show a problem when exiting, seen only with valgrind.
+      // Not clear why, as calling .FULL_RESET prior exiting does not show this error. 
+      all_procs[proc_name].reset(
+          new DLibPro((void (*)(EnvT * e)) (increment++), (void*) CallDllPro,  proc_name, max_args, min_args, has_keys),
+          CleanupProc
+          );
+      my_procs.insert(proc_name);
+      ADD_PROC_TO_INFOLIST_AND_INCREMENT_COUNTER
+    }
+    
+    void RegisterNativeProc( const string& lib_symbol, const string& proc_name, DLong max_args, DLong min_args, const string keyNames[]) { 
       if( all_procs.count( proc_name ) ) return;
       all_procs[proc_name].reset(
 				 new DLibPro( LinkAs<LibPro>( lib_symbol, proc_name ), proc_name.c_str(), max_args, keyNames, NULL, min_args),
@@ -180,25 +527,28 @@ namespace lib {
 				 );
       my_procs.insert(proc_name);
     }
-    void RegsisterFunc( const string& lib_symbol, const string& func_name, DLong max_args, DLong min_args, const string keyNames[] ) { 
+
+    void RegisterMediatizedFunc(const string& dlm_name, void* target, const string& func_name, DLong max_args, DLong min_args, bool has_keys) { 
+      if( all_funcs.count( func_name ) ) return;
+      // this method of 'cleaning' is excellent but show a problem when exiting, seen only with valgrind.
+      all_funcs[func_name].reset(
+				 new DLibFun((BaseGDL* (*)(EnvT* e)) (increment++), (void*)CallDllFunc , func_name, max_args,  min_args, has_keys),
+				 CleanupFunc
+				 );
+      my_funcs.insert(func_name);
+      ADD_PROC_TO_INFOLIST_AND_INCREMENT_COUNTER
+     }
+
+     void RegisterNativeFunc( const string& lib_symbol, const string& func_name, DLong max_args, DLong min_args,  string keyNames[]) { 
       if( all_funcs.count( func_name ) ) return;
       all_funcs[func_name].reset(
 				 new DLibFun( LinkAs<LibFun>( lib_symbol, func_name ), func_name.c_str(), max_args, keyNames, NULL, min_args),
 				 CleanupFunc
 				 );
       my_funcs.insert(func_name);
-    }
-    bool isLoaded( void ) { return handle; };
-        
-    handle_t handle;                                    // Handle to the linked DLL
-        
-    set<string> my_procs;                               // list of procedures linked within this DLL
-    set<string> my_funcs;                               // list of functions linked within this DLL
-        
-    static map<string,shared_ptr<DLibPro>> all_procs;   // list of ALL procedures linked (using linkimage)
-    static map<string,shared_ptr<DLibFun>> all_funcs;   // list of ALL functions linked (using linkimage)
-    static map<string,DllContainer> libs;               // list of ALL DLLs linked (using linkimage)
-        
+      }
+  
+    bool isLoaded( void ) { return handle; };        
   };
 
   // Instantiate static members
@@ -206,7 +556,6 @@ namespace lib {
   map<string,shared_ptr<DLibFun>> DllContainer::all_funcs;
   map<string,DllContainer> DllContainer::libs;
 
-
   void point_lun( EnvT* e) 
   { 
     e->NParam( 1);//, "POINT_LUN");
@@ -364,13 +713,62 @@ namespace lib {
       if (docount) (*returnedCount)[0] = ret;
     }
   }
+  void dlm_load( EnvT* e ) {
+    // do nothing as long we do load the .so segment when opening a dlm.
+    // if not, this would be a good place to call DllContainer::get( shrdimgName ); (see below)
+  }
+  
+  //linkimage is used by all DLM-related stuff, but should behave differently when called 'a la DLM'.
+  //Hence the use of DLM_INFO string array.
+  
+#define COPYSTR(what)      IDL_SysvVersion.what.slen=what.length();\
+  IDL_SysvVersion.what.s=(char*)malloc(what.length());\
+  strncpy(IDL_SysvVersion.what.s,what.c_str(),what.length());
+ 
   void linkimage( EnvT* e ) {
-
+    
+    static bool IdlStaticsUninitialized=true;
+    if (IdlStaticsUninitialized) {
+      IdlStaticsUninitialized=false;
+      //set up values for some EXPORT_XXX info structures
+      //EXPORT_SysvVersion:
+      DStructGDL* version = SysVar::Version();
+      static unsigned releaseTag = version->Desc()->TagIndex( "RELEASE");
+      static unsigned osTag = version->Desc()->TagIndex("OS");
+      static unsigned osFamilyTag = version->Desc()->TagIndex("OS_FAMILY");
+      static unsigned osNameTag = version->Desc()->TagIndex("OS_NAME");
+      static unsigned osBuildTag = version->Desc()->TagIndex("BUILD_DATE");
+      static unsigned archTag = version->Desc()->TagIndex("ARCH");
+      static unsigned mTag = version->Desc()->TagIndex("MEMORY_BITS");
+      static unsigned foTag = version->Desc()->TagIndex("FILE_OFFSET_BITS");
+      DString arch =     (*static_cast<DStringGDL*> (version->GetTag(archTag, 0)))[0];
+      COPYSTR(arch)
+      DString os =       (*static_cast<DStringGDL*> (version->GetTag(osTag, 0)))[0];
+      COPYSTR(os)
+      DString os_family =       (*static_cast<DStringGDL*> (version->GetTag(osFamilyTag, 0)))[0];
+      COPYSTR(os_family)
+      DString os_name =       (*static_cast<DStringGDL*> (version->GetTag(osNameTag, 0)))[0];
+      COPYSTR(os_name)
+      DString release =  (*static_cast<DStringGDL*>( version->GetTag( releaseTag, 0)))[0];
+      COPYSTR(release)
+      DString build_date =       (*static_cast<DStringGDL*> (version->GetTag(osBuildTag, 0)))[0];
+      COPYSTR(build_date)
+      DInt memory_bits =       (*static_cast<DIntGDL*> (version->GetTag(mTag, 0)))[0]; 
+      IDL_SysvVersion.memory_bits=memory_bits;
+      DInt file_offset_bits =       (*static_cast<DIntGDL*> (version->GetTag(foTag, 0)))[0]; 
+      IDL_SysvVersion.file_offset_bits=file_offset_bits;
+      //IDL_SysvDir:
+      DString dir=SysVar::Dir();
+      IDL_SysvDir.slen=dir.length();
+      IDL_SysvDir.s=(char*)malloc(dir.length());
+      strncpy(IDL_SysvDir.s,dir.c_str(),dir.length());
+    }
+#undef COPYSTR
+    
     SizeT nP = e->NParam(2);
  
     DString funcName;
     e->AssureScalarPar<DStringGDL>( 0, funcName );
-    DString upCasefuncName = StrUpCase( funcName );
 
     DString shrdimgName;
     e->AssureScalarPar<DStringGDL>( 1, shrdimgName );
@@ -389,44 +787,69 @@ namespace lib {
     static int keywordsIx = e->KeywordIx("KEYWORDS");
     static int maxargsIx = e->KeywordIx("MAX_ARGS");
     static int minargsIx = e->KeywordIx("MIN_ARGS");
-    
-    if( e->KeywordPresent( functIx ) ) funcType = 1;
+    static int nativeIx = e->KeywordIx("NATIVE");
+    bool isNativeGdl=e->KeywordSet(nativeIx); //this is a GDL-native .so 
+    static int dlminfoIx = e->KeywordIx("DLM_INFO");
+    bool isDlm=e->KeywordSet(dlminfoIx);
+
+    if( e->KeywordSet( functIx ) ) funcType = 1;
     
     DLong max_args = 16;
     e->AssureLongScalarKWIfPresent( maxargsIx, max_args );
     DLong min_args = 0;
     e->AssureLongScalarKWIfPresent( minargsIx, min_args );
 
-    vector<string> keywords;
-    string* kw_ptr = nullptr;
-    if( e->KeywordSet( keywordsIx ) ) {
-      DStringGDL* kws = e->GetKWAs<DStringGDL>( keywordsIx );
-      if( kws ) {
-	SizeT nEL = kws->N_Elements();
-	for( SizeT i=0; i<nEL; ++i ) {
-	  string tmpS = StrUpCase((*kws)[i]);
-	  if( !tmpS.empty() ) keywords.push_back( tmpS );
-	}
-      }
-    }
-
-    if( !keywords.empty() ) {
-      std::sort( keywords.begin(), keywords.end() ); 
-      keywords.push_back("");
-      kw_ptr = keywords.data();
-    }
-
+    bool hasKeywords=( e->KeywordSet( keywordsIx ) );
+    //if 'native' (gdl) then if keywords, these are a DStringGDL
+    
     if( entryName.empty() ) {
       entryName = funcName;
     }
 
-    try {
-      DllContainer& lib = DllContainer::get( shrdimgName );
-      lib.RegsisterSymbol( entryName, upCasefuncName, funcType, max_args, min_args, kw_ptr );
-    } catch ( const std::exception& ex ) {
-      e->Throw("Error linking procedure/DLL: " + funcName + " -> " + entryName + "  (" + shrdimgName + ") : " + ex.what() );
-    }
-    
+//    try {
+      DStringGDL* info = NULL;
+      if (isDlm) info = e->GetKWAs<DStringGDL>(dlminfoIx);
+      DllContainer& lib = DllContainer::get(shrdimgName);
+      if (lib.IsOK()) {
+        if (isNativeGdl) {
+          //if 'native' (gdl) then if keywords, these are a DStringGDL
+          if (hasKeywords) {
+            BaseGDL* p = e->GetKW(keywordsIx);
+            if (p->Type() != GDL_STRING) e->Throw("KEYWORDS: Expecting STRING keywords.");
+            // GD: attention Keyword list must be null terminated!
+            DStringGDL* crude = e->GetKWAs<DStringGDL>(keywordsIx);
+            DStringGDL* null_terminated = new DStringGDL(dimension(crude->N_Elements() + 1));
+            for (auto i = 0; i < p->N_Elements(); ++i) (*null_terminated)[i] = (*crude)[i];
+            lib.RegisterNativeSymbol(entryName, funcName, funcType, max_args, min_args, &((*null_terminated)[0]));
+          } else lib.RegisterNativeSymbol(entryName, funcName, funcType, max_args, min_args);
+        } else {
+          int ret = 0;
+          if (isDlm) {
+            // test if "IDL_Load() can be called to give, noot symbol names, but symbols addresses
+            try {
+              ret = lib.CallLoadToDefineEntryLocations();
+            } catch (GDLException ex) { Warning((*info)[0]+": Recovering from error: "+ex.toString()); e->Interpreter()->RetAll();}
+            if (ret) { //if we got directly symbol addresses, use a special function
+              lib.RegisterSymbolDefinedByIDL_Load((*info)[0],funcName, funcType, max_args, min_args, hasKeywords);
+              if (!lib.IsHelpAdded()) { //can still be unloaded due to problems.
+                help_AddDlmInfo(info->Dup()); //only once
+                lib.HelpAdded(true);
+              }
+              if (funcType) {//insert DLM name in lib/pro structure to enable the "Loaded XXXX" message at first use.
+                int funIx=LibFunIx(funcName);
+                } else {
+                int proIx=LibProIx(funcName);
+                }
+            } else lib.RegisterSymbol((*info)[0], entryName, funcName, funcType, max_args, min_args, hasKeywords); //not a dlm, or dlm does not have ILD_Load()
+          }
+        }
+      } else {
+        e->Throw("Error linking procedure/DLL: " + funcName + " -> " + entryName + "  (" + shrdimgName + ") : " +lib.DllLoadErrorMessage); e->Interpreter()->RetAll();}
+//    } catch (GDLException ex) {
+//      e->Throw(ex.toString());
+//    } catch (const std::exception& ex) {
+//      e->Throw("Error linking procedure/DLL: " + funcName + " -> " + entryName + "  (" + shrdimgName + ") : " + ex.what());
+//    } catch (...) { e->Throw("Unexpected exception.");}
   }
   
   void unlinkimage( EnvT* e ) {
@@ -470,7 +893,7 @@ namespace lib {
 
     try {
       DllContainer& lib = DllContainer::get( shrdimgName );
-      lib.UnregsisterSymbol( upCasefuncName, funcType );
+      lib.UnregisterSymbol( upCasefuncName, funcType );
       lib.unload();       // will only unload if all symbols have been unregistered.
     } catch ( const std::exception& ex ) {
       e->Throw("Error unlinksymbol: " + funcName + "  (" + shrdimgName + ") : " + ex.what() );
@@ -483,6 +906,76 @@ namespace lib {
     DllContainer::clear();
   }
   
+  //idem WAIT_PRO, but needed in idlneturl as we wait while some widgets events must happen.
+  //should NEVER be used elsewhere (the idlneturl.pro procedure should disappear in favor of a pure C/C++ use of the CURL library)
+  //The real WAIT blocks absolutely anything --- as it should.
+  void gdlwait_responsive(EnvT* e) {
+    e->NParam(1); //, "WAIT");
+
+    DDouble waittime;
+    e->AssureDoubleScalarPar(0, waittime);
+
+    if (waittime < 0)
+      throw GDLException(e->CallingNode(),
+        "WAIT: Argument must be non-negative"
+        + e->GetParString(0));
+
+#ifdef _WIN32
+    LARGE_INTEGER Frequency;
+    LARGE_INTEGER BeginTime;
+    LARGE_INTEGER Endtime;
+    LARGE_INTEGER elapsed;
+    LARGE_INTEGER waittime_us;
+    waittime_us.QuadPart = waittime * 1e6;
+
+    QueryPerformanceFrequency(&Frequency);
+    QueryPerformanceCounter(&BeginTime);
+
+    while (1) {
+      QueryPerformanceCounter(&Endtime);
+      elapsed.QuadPart = (Endtime.QuadPart - BeginTime.QuadPart) / (Frequency.QuadPart / 1000000);
+      if (elapsed.QuadPart >= waittime_us.QuadPart) break;
+      else if (elapsed.QuadPart > 100) Sleep(80);
+    }
+#else
+    int old_version = 0;
+
+    if (waittime <= 0.005) old_version = 1;
+
+    // AC 2010-09-16
+    // this version is OK and very accurate for small durations
+    // but used 100% of one CPU :((
+    if (old_version == 1) {
+      struct timeval tval;
+      struct timezone tzone;
+
+      // derivated from the current version of SYSTIME()
+      gettimeofday(&tval, &tzone);
+      double t_start = tval.tv_sec + tval.tv_usec / 1e+6; // time in UTC seconds
+      double t_current = 0.0;
+
+      double diff = 0.0;
+      while (diff < waittime) {
+
+        gettimeofday(&tval, &tzone);
+        t_current = tval.tv_sec + tval.tv_usec / 1e+6;
+        diff = t_current - t_start;
+      }
+    }
+
+    // AC 2010-09-16 this version should used much less CPU !
+    if (old_version == 0) {
+      //cout << floor(waittime) << " " <<  waittime-floor(waittime) << endl;
+      struct timespec tv;
+      tv.tv_sec = floor(waittime);
+      tv.tv_nsec = (waittime - floor(waittime))*1e9;
+      int retval;
+      retval = nanosleep(&tv, NULL);
+    }
+#endif
+   GDLEventHandler(); //this is probably not OK, but is at the moment needed to permit delivery of callback functions in idlneturl__define.pro where a waiting loop uses teh WAIT command.  
+  }
+  
   void wait_pro( EnvT* e) 
   { 
     e->NParam( 1);//, "WAIT");
@@ -792,10 +1285,7 @@ namespace lib {
     HMODULE handle = LoadLibraryW(wchr);
     delete(wchr);
 #else
-    // note following code line is correct for gcc and linux.
-    // in case of trouble on some architecture, find the correct option and make an #ifdef.
-    // Do *not* modify the following line. GD.
-    void* handle =  dlopen(image.c_str(),  RTLD_NOW | RTLD_GLOBAL);
+    void* handle =  dlopen(image.c_str(), RTLD_LAZY);
 #endif
     if (handle == NULL) {
 #if !defined(_WIN32) || defined(__CYGWIN__)
