@@ -31,10 +31,12 @@ namespace lib {
   void copy_lun( EnvT* e );
   void truncate_lun( EnvT* e );
   void linkimage( EnvT* e );
+  void dlm_load( EnvT* e );
   void unlinkimage( EnvT* e );
   void unlinksymbol( EnvT* e );
   void ResetDLLs( void );
   void wait_pro( EnvT* e);
+  void gdlwait_responsive( EnvT* e);
 
   void kwtest( EnvT* e);
 
