@@ -77,7 +77,7 @@ public:
   char* NoReadline(const std::string&);
 
 private:
-
+  bool inInnerInterpreterLoop=false;
   // execute GDL command (.run, .step, ...)
   CommandCode ExecuteCommand(const std::string& command);
   CommandCode CmdCompile(const std::string& command);
@@ -107,6 +107,9 @@ public:
   // execute one line of code
   CommandCode ExecuteLine( std::istream* in = NULL, SizeT lineOffset = 0);
 
+// execute one line of code
+  CommandCode ExecuteStringLine( std::string &line);
+
   // execute a whole file (used by @ and for batch files specified as arguments to gdl)
   void        ExecuteFile( const std::string& file);  
 
@@ -121,6 +124,8 @@ public:
   RetCode InnerInterpreterLoop(SizeT lineOffset);
 
   bool IsInBatchProcedureAtMain(){return InBatchProcedureAtMain;}
+  bool IsInnerInterpreterLoop(){ return inInnerInterpreterLoop;}
+  void SetInnerInterpeterLoop(bool b){inInnerInterpreterLoop=b;}
 };
 
 #endif
