@@ -199,7 +199,7 @@ public:
   int   NParMin() const { return nParMin;}
 
   //  bool AKey() { return aKey;} // additional keywords allowed
-
+  
   friend class EnvBaseT;
   friend class EnvT;
   friend class ExtraT;
@@ -218,11 +218,13 @@ public:
 class DLib: public DSub
 {
   bool hideHelp; // if set HELP,/LIB will not list this subroutine
-  
+
 public:
+  void* mediator;
+  
   DLib( const std::string& n, const std::string& o, const int nPar_,
 	const std::string keyNames[],
-	const std::string warnKeyNames[], const int nParMin_, const bool use_threadpool=false);
+	const std::string warnKeyNames[], const int nParMin_, const bool use_threadpool=false, void* mediator=NULL, bool usesKeywords=false);
 
   virtual const std::string ToString() = 0;
   
@@ -250,6 +252,8 @@ public:
   // change the results.
   // Note that due to their nature, there should never be keywords
   // on which a value is returned.
+  DLibPro( LibPro p, void* mediator, const std::string& n, const int nPar_=0, const int nParMin_=0, const bool hasKeys=false);
+
   DLibPro( LibPro p, const std::string& n, const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
 	   const std::string warnKeyNames[]=NULL, const int nParMin_=0, const bool use_threadpool=false);
@@ -258,9 +262,9 @@ public:
 	   const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
 	   const std::string warnKeyNames[]=NULL, const int nParMin_=0);
-
-  LibPro Pro() { return pro;}
-
+  void* GetProAddress() {return (void*) pro;} //pro address MAY NOT BE a real Procedure address.
+  LibPro Pro() { if (mediator) return (LibPro)mediator; else return pro;}
+  
   const std::string ToString();
 };
 
@@ -269,6 +273,8 @@ class DLibFun: public DLib
 {
   LibFun fun;
 public:
+  DLibFun( LibFun f, void* mediator, const std::string& n, const int nPar_=0, const int nParMin_=0, const bool hasKeys=false);
+
   DLibFun( LibFun f, const std::string& n, const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
 	   const std::string warnKeyNames[]=NULL, const int nParMin_=0, const bool use_threadpool=false);
@@ -278,7 +284,8 @@ public:
 	   const std::string keyNames[]=NULL,
 	   const std::string warnKeyNames[]=NULL, const int nParMin_=0);
 
-  LibFun Fun() { return fun;}
+  void* GetFunAddress() {return (void*) fun;} //fun address MAY NOT BE a real Function address.
+  LibFun Fun() { if (mediator) return (LibFun)mediator; else return fun;}
 
   const std::string ToString();
 
@@ -318,7 +325,7 @@ public:
   bool RetConstant() { return this->retConstant;}
 };
 // direct call functions must have:
-// ony one parameter, no keywords
+// only one parameter, no keywords
 // these functions are called "direct", no environment is created
 class DLibFunDirect: public DLibFunRetNew
 {
