@@ -23,9 +23,8 @@ namespace SysVar
 {
   //  extern unsigned int pathIx, dIx;
   DFloat* GetSC();
-  // returns a StrArr with the path to search
-  const StrArr& GDLPath();
-  void SetGDLPath( const DString& newPath);
+  void SetGDLPath( DString& newPath);
+  void SetDlmPath( DString& newPath);
 
   // returns !P
   DStructGDL* P();
@@ -93,15 +92,12 @@ namespace SysVar
   void Suppress_Edit_Input();
   DLong Quiet();
   void Make_Quiet();
-  DLong GDL_Warning();
-
+  DInt More();
+  
   // get and set !JOURNAL (journal file LUN)
   DLong JournalLUN();
   void JournalLUN( DLong jLUN);
 
-  // get and set USERSYM
-  DStructGDL* USYM();
-
   // get the !ORDER variable
   DLong TV_ORDER();
   
