@@ -45,12 +45,14 @@ namespace lib {
     switch (vartype)
       {
       case NC_BYTE:  return DStringGDL("BYTE");//8 bit
-      case NC_UBYTE:  return DStringGDL("UBYTE");//8 bit
+      case NC_UBYTE: return DStringGDL("UBYTE");//8 bit
       case NC_CHAR:  return DStringGDL("CHAR");//8 bit as string
       case NC_SHORT: return DStringGDL("INT");//16 bit
-      case NC_USHORT: return DStringGDL("UINT");//16 bit
+      case NC_USHORT:return DStringGDL("UINT");//16 bit
       case NC_INT:   return DStringGDL("LONG");//32 bit
-      case NC_UINT:   return DStringGDL("ULONG");//32 bit
+      case NC_UINT:  return DStringGDL("ULONG");//32 bit
+      case NC_INT64: return DStringGDL("LONG64");//64 bit
+      case NC_UINT64:return DStringGDL("ULONG64");//64 bit
       case NC_FLOAT: return DStringGDL("FLOAT");//32 bit float
       case NC_DOUBLE:return DStringGDL("DOUBLE");//64 bit double
       case NC_STRING:return DStringGDL("STRING");//String
@@ -303,7 +305,7 @@ namespace lib {
     
     if (status != 0) {
       
-      if ((status == -31) | (status == 2)) 
+      if ((status == -31) || (status == 2)) 
 	{
 	  // before any NetCDF stuff, we check whether the file exists ...
 	  if (!is_readable(s))
