@@ -41,6 +41,7 @@
 #include "str.hpp"
 #include "gdlexception.hpp"
 #include "initsysvar.hpp" // GDLPath();
+
 namespace lib {
   std::string PathSeparator()
   {
@@ -70,12 +71,16 @@ namespace lib {
 //namespace lib {
 //bool trace_arg();
 //}
+
+//GetEnvString and GetEnvPathString should be able to filter  <IDL_DEFAULT> like things 
+
 std::string GetEnvString(const char* env)
 {
   char* c=getenv(env);
   if( !c) return std::string("");
   return std::string(c);
 }
+
 //same but for a path: separator will be '\' for _WIN32
 std::string GetEnvPathString(const char* env)
 {
@@ -416,7 +421,7 @@ void WordExp(std::string& s)
   //AC 2018-04-25 : because crash of :  // openr, unit, '', ERROR=error,/get_lun
   if (s.length() == 0) return;
   bool trace_me = false; //lib::trace_arg();
-
+  
   std::string sEsc = "";
   int ipos = 0;
   // escape blanks
@@ -531,8 +536,7 @@ bool CompleteFileName(std::string& fn)
 
   if( PathGiven(fn)) return false;
 
-  StrArr path=SysVar::GDLPath();
-  if( path.size() == 0) 
+  if( CurrentPathList.size() == 0) 
   {
     std::string act = "./pro/"; // default path if no path is given
 
@@ -551,9 +555,9 @@ bool CompleteFileName(std::string& fn)
     }
   }
   else
-    for(unsigned p=0; p<path.size(); p++)
+    for(unsigned p=0; p<CurrentPathList.size(); p++)
       {
-	std::string act=path[p];
+	std::string act=CurrentPathList[p];
 	
 	AppendIfNeeded(act,lib::PathSeparator());
 	
